@@ -17,6 +17,7 @@
 from __future__ import unicode_literals
 
 import logging
+import warnings
 
 import mock
 from oslotest import base as test_base
@@ -84,8 +85,8 @@ class MessageTestCase(test_base.BaseTestCase):
         result = _message.Message(msgid) % params
 
         expected = msgid % params
-        self.assertEqual(result, expected)
-        self.assertEqual(result.translate(), expected)
+        self.assertEqual(expected, result)
+        self.assertEqual(expected, result.translate())
 
     def test_multiple_mod_with_named_parameter(self):
         msgid = ("%(description)s\nCommand: %(cmd)s\n"
@@ -144,8 +145,8 @@ class MessageTestCase(test_base.BaseTestCase):
         result = _message.Message(msgid) % params
 
         expected = msgid % params
-        self.assertEqual(result, expected)
-        self.assertEqual(result.translate(), expected)
+        self.assertEqual(expected, result)
+        self.assertEqual(expected, result.translate())
 
     def test_mod_with_dict_parameter(self):
         msgid = "Test that we can inject a dictionary %s"
@@ -157,6 +158,40 @@ class MessageTestCase(test_base.BaseTestCase):
         self.assertEqual(expected, result)
         self.assertEqual(expected, result.translate())
 
+    def test_mod_with_wrong_field_type_in_trans(self):
+        msgid = "Correct type %(arg1)s"
+        params = {'arg1': 'test1'}
+        with mock.patch('gettext.translation') as trans:
+            # Set up ugettext to return the original message with the
+            # correct format string.
+            trans.return_value.ugettext.return_value = msgid
+            # Build a message and give it some parameters.
+            result = _message.Message(msgid) % params
+            # Now set up ugettext to return the translated version of
+            # the original message, with a bad format string.
+            wrong_type = u'Wrong type %(arg1)d'
+            if six.PY3:
+                trans.return_value.gettext.return_value = wrong_type
+            else:
+                trans.return_value.ugettext.return_value = wrong_type
+            trans_result = result.translate()
+            expected = msgid % params
+            self.assertEqual(expected, trans_result)
+
+    def test_mod_with_wrong_field_type(self):
+        msgid = "Test that we handle unused args %(arg1)d"
+        params = {'arg1': 'test1'}
+
+        with testtools.ExpectedException(TypeError):
+            _message.Message(msgid) % params
+
+    def test_mod_with_missing_arg(self):
+        msgid = "Test that we handle missing args %(arg1)s %(arg2)s"
+        params = {'arg1': 'test1'}
+
+        with testtools.ExpectedException(KeyError, '.*arg2.*'):
+            _message.Message(msgid) % params
+
     def test_mod_with_integer_parameters(self):
         msgid = "Some string with params: %d"
         params = [0, 1, 10, 24124]
@@ -169,12 +204,12 @@ class MessageTestCase(test_base.BaseTestCase):
 
         for message, result in zip(messages, results):
             self.assertEqual(type(result), _message.Message)
-            self.assertEqual(result.translate(), message)
+            self.assertEqual(message, result.translate())
 
             # simulate writing out as string
             result_str = '%s' % result.translate()
             self.assertEqual(result_str, message)
-            self.assertEqual(result, message)
+            self.assertEqual(message, result)
 
     def test_mod_copies_parameters(self):
         msgid = "Found object: %(current_value)s"
@@ -185,7 +220,7 @@ class MessageTestCase(test_base.BaseTestCase):
         changing_dict['current_value'] = 2
         # Even if the param changes when the message is
         # translated it should use the original param
-        self.assertEqual(result.translate(), 'Found object: 1')
+        self.assertEqual('Found object: 1', result.translate())
 
     def test_mod_deep_copies_parameters(self):
         msgid = "Found list: %(current_list)s"
@@ -197,26 +232,26 @@ class MessageTestCase(test_base.BaseTestCase):
         changing_list.append(4)
         # Even though the list changed the message
         # translation should use the original list
-        self.assertEqual(result.translate(), "Found list: [1, 2, 3]")
+        self.assertEqual("Found list: [1, 2, 3]", result.translate())
 
     def test_mod_deep_copies_param_nodeep_param(self):
         msgid = "Value: %s"
         params = utils.NoDeepCopyObject(5)
         # Apply the params
         result = _message.Message(msgid) % params
-        self.assertEqual(result.translate(), "Value: 5")
+        self.assertEqual("Value: 5", result.translate())
 
     def test_mod_deep_copies_param_nodeep_dict(self):
         msgid = "Values: %(val1)s %(val2)s"
         params = {'val1': 1, 'val2': utils.NoDeepCopyObject(2)}
         # Apply the params
         result = _message.Message(msgid) % params
-        self.assertEqual(result.translate(), "Values: 1 2")
+        self.assertEqual("Values: 1 2", result.translate())
 
         # Apply again to make sure other path works as well
         params = {'val1': 3, 'val2': utils.NoDeepCopyObject(4)}
         result = _message.Message(msgid) % params
-        self.assertEqual(result.translate(), "Values: 3 4")
+        self.assertEqual("Values: 3 4", result.translate())
 
     def test_mod_returns_a_copy(self):
         msgid = "Some msgid string: %(test1)s %(test2)s"
@@ -252,21 +287,11 @@ class MessageTestCase(test_base.BaseTestCase):
         result = _message.Message(msgid) % params
 
         expected = msgid % params
-        self.assertEqual(result, expected)
-        self.assertEqual(result.translate(), expected)
+        self.assertEqual(expected, result)
+        self.assertEqual(expected, result.translate())
 
         # Make sure unused params still there
-        self.assertEqual(result.params.keys(), params.keys())
-
-    def test_mod_with_missing_named_parameters(self):
-        msgid = ("Some string with params: %(param1)s %(param2)s"
-                 " and a missing one %(missing)s")
-        params = {'param1': 'test',
-                  'param2': 'test2'}
-
-        test_me = lambda: _message.Message(msgid) % params
-        # Just like with strings missing named parameters raise KeyError
-        self.assertRaises(KeyError, test_me)
+        self.assertEqual(params.keys(), result.params.keys())
 
     def test_add_disabled(self):
         msgid = "A message"
@@ -278,12 +303,6 @@ class MessageTestCase(test_base.BaseTestCase):
         test_me = lambda: utils.SomeObject('test') + _message.Message(msgid)
         self.assertRaises(TypeError, test_me)
 
-    @testtools.skipIf(six.PY3, 'test specific to Python 2')
-    def test_str_disabled(self):
-        msgid = "A message"
-        test_me = lambda: str(_message.Message(msgid))
-        self.assertRaises(UnicodeError, test_me)
-
     @mock.patch('gettext.translation')
     def test_translate(self, mock_translation):
         en_message = 'A message in the default locale'
@@ -352,6 +371,86 @@ class MessageTestCase(test_base.BaseTestCase):
         self.assertEqual(expected_translation, msg.translate('es'))
         self.assertEqual(default_translation, msg.translate('XX'))
 
+    @mock.patch('gettext.translation')
+    @mock.patch('oslo_i18n._message.LOG')
+    def test_translate_message_bad_translation(self,
+                                               mock_log,
+                                               mock_translation):
+        message_with_params = 'A message: %s'
+        es_translation = 'A message in Spanish: %s %s'
+        param = 'A Message param'
+
+        translations = {message_with_params: es_translation}
+        translator = fakes.FakeTranslations.translator({'es': translations})
+        mock_translation.side_effect = translator
+
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always")
+            msg = _message.Message(message_with_params)
+            msg = msg % param
+            default_translation = message_with_params % param
+
+            self.assertEqual(default_translation, msg.translate('es'))
+
+            self.assertEqual(1, len(w))
+            # Note(gibi): in python 3.4 str.__repr__ does not put the unicode
+            # marker 'u' in front of the string representations so the test
+            # removes that to have the same result in python 2.7 and 3.4
+            self.assertEqual("Failed to insert replacement values into "
+                             "translated message A message in Spanish: %s %s "
+                             "(Original: 'A message: %s'): "
+                             "not enough arguments for format string",
+                             str(w[0].message).replace("u'", "'"))
+
+        mock_log.debug.assert_called_with(('Failed to insert replacement '
+                                           'values into translated message '
+                                           '%s (Original: %r): %s'),
+                                          es_translation,
+                                          message_with_params,
+                                          mock.ANY)
+
+    @mock.patch('gettext.translation')
+    @mock.patch('locale.getdefaultlocale', return_value=('es', ''))
+    @mock.patch('oslo_i18n._message.LOG')
+    def test_translate_message_bad_default_translation(self,
+                                                       mock_log,
+                                                       mock_local,
+                                                       mock_translation):
+        message_with_params = 'A message: %s'
+        es_translation = 'A message in Spanish: %s %s'
+        param = 'A Message param'
+
+        translations = {message_with_params: es_translation}
+        translator = fakes.FakeTranslations.translator({'es': translations})
+        mock_translation.side_effect = translator
+
+        msg = _message.Message(message_with_params)
+
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always")
+            msg = msg % param
+            self.assertEqual(1, len(w))
+            # Note(gibi): in python 3.4 str.__repr__ does not put the unicode
+            # marker 'u' in front of the string representations so the test
+            # removes that to have the same result in python 2.7 and 3.4
+            self.assertEqual("Failed to insert replacement values into "
+                             "translated message A message in Spanish: %s %s "
+                             "(Original: 'A message: %s'): "
+                             "not enough arguments for format string",
+                             str(w[0].message).replace("u'", "'"))
+
+        mock_log.debug.assert_called_with(('Failed to insert replacement '
+                                           'values into translated message '
+                                           '%s (Original: %r): %s'),
+                                          es_translation,
+                                          message_with_params,
+                                          mock.ANY)
+        mock_log.reset_mock()
+
+        default_translation = message_with_params % param
+        self.assertEqual(default_translation, msg)
+        self.assertFalse(mock_log.warning.called)
+
     @mock.patch('gettext.translation')
     def test_translate_message_with_object_param(self, mock_translation):
         message_with_params = 'A message: %s'
@@ -516,3 +615,73 @@ class MessageTestCase(test_base.BaseTestCase):
         # Translation into other locales still works
         self.assertEqual(zh_translation, msg.translate('zh'))
         self.assertEqual(fr_translation, msg.translate('fr'))
+
+
+class TranslateMsgidTest(test_base.BaseTestCase):
+
+    @mock.patch('gettext.translation')
+    def test_contextual(self, translation):
+        lang = mock.Mock()
+        translation.return_value = lang
+        trans = mock.Mock()
+        trans.return_value = 'translated'
+        lang.gettext = trans
+        lang.ugettext = trans
+        result = _message.Message._translate_msgid(
+            ('context', 'message'),
+            domain='domain',
+            has_contextual_form=True,
+            has_plural_form=False,
+        )
+        self.assertEqual('translated', result)
+        trans.assert_called_with(
+            'context' + _message.CONTEXT_SEPARATOR + 'message'
+        )
+
+    @mock.patch('gettext.translation')
+    def test_contextual_untranslatable(self, translation):
+        msg_with_context = 'context' + _message.CONTEXT_SEPARATOR + 'message'
+        lang = mock.Mock()
+        translation.return_value = lang
+        trans = mock.Mock()
+        trans.return_value = msg_with_context
+        lang.gettext = trans
+        lang.ugettext = trans
+        result = _message.Message._translate_msgid(
+            ('context', 'message'),
+            domain='domain',
+            has_contextual_form=True,
+            has_plural_form=False,
+        )
+        self.assertEqual('message', result)
+        trans.assert_called_with(msg_with_context)
+
+    @mock.patch('gettext.translation')
+    def test_plural(self, translation):
+        lang = mock.Mock()
+        translation.return_value = lang
+        trans = mock.Mock()
+        trans.return_value = 'translated'
+        lang.ngettext = trans
+        lang.ungettext = trans
+        result = _message.Message._translate_msgid(
+            ('single', 'plural', -1),
+            domain='domain',
+            has_contextual_form=False,
+            has_plural_form=True,
+        )
+        self.assertEqual('translated', result)
+        trans.assert_called_with(
+            'single', 'plural', -1,
+        )
+
+    @mock.patch('gettext.translation')
+    def test_contextual_and_plural(self, translation):
+        self.assertRaises(
+            ValueError,
+            _message.Message._translate_msgid,
+            'nothing',
+            domain='domain',
+            has_contextual_form=True,
+            has_plural_form=True,
+        )
